/*
 *    FBFS Distributed Object Repository
 *    Copyright (C) 2001 Francesco V. Bassi
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <iostream.h>
#include "../Echo.hh"
#include "../../utils/NameServer.h"

using namespace fbfs;

static void hello(Echo_ptr e)
{
	if( CORBA::is_nil(e) ) {
		cerr << "hello: The object reference is nil!\n" << endl;
		return;
	}

	CORBA::String_var dest = e->echoString();

	cerr << "The Echo object replied, \"" << (char*)dest <<"\"." << endl;
}

//////////////////////////////////////////////////////////////////////

int
main (int argc, char **argv) 
{
	try {
		if(argc != 2)
			return 0;
		CORBA::ORB_var orb = CORBA::ORB_init(argc, argv, "omniORB3");

		utils::URI uri = utils::URI(argv[1]);

		cout << uri.toString() << "\n";
		CORBA::Object_var obj = utils::NameServer::get_object(orb, uri);

		Echo_var echoref = Echo::_narrow(obj);
		hello(echoref);

		orb->destroy();
	}
	catch(CORBA::COMM_FAILURE& ex) {
		cerr << "Caught system exception COMM_FAILURE -- unable to contact the "
		 << "object." << endl;
	}
	catch(CORBA::SystemException&) {
		cerr << "Caught CORBA::SystemException." << endl;
	}
	catch(CORBA::Exception&) {
		cerr << "Caught CORBA::Exception." << endl;
	}
	catch(omniORB::fatalException& fe) {
		cerr << "Caught omniORB::fatalException:" << endl;
		cerr << "  file: " << fe.file() << endl;
		cerr << "  line: " << fe.line() << endl;
		cerr << "  mesg: " << fe.errmsg() << endl;
	}
	catch(...) {
		cerr << "Caught unknown exception." << endl;
	}
	return 0;
}
