/*
 *    FBFS Distributed Object Repository
 *    Copyright (C) 2001 Francesco V. Bassi
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/** \file NameServerClient.cc
 * Implementazione dei metodi della classe fbfs::NameServerClient
 * \ingroup NameServer
 */
   
#include "NameServerClient.h"
#include "../idl/Security.hh"
#include "../utils/NameServer.h"
#include "../idl/NameServer.hh"

fbfs::NameServerClient::NameServerClient(const CORBA::ORB_ptr &o, const utils::Prefs &p,
	const string &pri, const string &sec)
	:	orb(o),
		primary(p.get("nameserver1", pri)),
		secondary(p.get("nameserver2", sec)) {
	};

utils::URI fbfs::NameServerClient::lookup(const fbfs::Obj_id &id) {
	utils::URI uri = lookup_int(id, primary);
	if(!uri.isValid())
		uri = lookup_int(id, secondary);
	return uri;
};

utils::URI fbfs::NameServerClient::lookup_int(const fbfs::Obj_id &id, utils::URI uri) const {
	if(!uri.isValid())
		return utils::URI("");
		
	try{
		CORBA::String_var dest;
		do {
			CORBA::Object_var obj = utils::NameServer::get_object(orb, uri);
			NameServer_var ns = NameServer::_narrow(obj);
			if( CORBA::is_nil(ns) )
				throw NotFound();
			dest = ns->lookup(cert, id);
			uri = utils::URI(string(dest));
		} while ( uri.isValid() && uri.service().compare("NameServer") == 0 );
		return uri;
	} catch (...) {
		return utils::URI("");
	};
};
