/*
 *    FBFS Distributed Object Repository
 *    Copyright (C) 2001 Francesco V. Bassi
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/** \file NameServer_i.cc
 * Implementazione dei metodi della classe fbfs::NameServer_i
 * \ingroup NameServer
 */
 
#include "NameServer_i.h"

#include <fstream.h>

using namespace fbfs;
using namespace utils;

fbfs::NameServer_i::NameServer_i(Prefs &p, const CORBA::ORB_ptr &o) : prefs(p), orb(o) {
	string rulefile = p.get("rulefile","nameserver-rules");

	ifstream::ifstream s;
	s.open(rulefile.c_str());
	char buf[80];
	while(!s.eof()) {
		s.getline(buf,80);
		string s(buf);
		if(s.find("#")==0 || s.length() < 3)
			continue;
		int level = 0;
		while(s[level++]==' ');
		level--;
		string line = s.substr(level,s.length()-level);
		int i;
		i = line.find(" ");
		string command = line.substr(0,i);
		line = line.substr(i+1,line.length()-i-1);
		i = line.find(" ");
		string arg = line.substr(0,i);

		Rule r(level, command, arg);
		rules.push_back(r);
	};
};
	
char *fbfs::NameServer_i::lookup(const Security &cert, const Obj_id &id)
            throw(NotFound, SecurityException, InternalError) {
	string name(id.name);
	string result;
	int level = 0;
	bool go_deeper = true;
	bool exit_now = false;
	vector<Rule>::iterator i;
	for(i=rules.begin(); i != rules.end(); i++) {
		if(i->level <= level || go_deeper) {
			level = i->level;
			switch( i->eval(name, result, orb) ) {
				case Rule::FALSE:
					go_deeper = false;
					break;
				case Rule::TRUE:
					go_deeper = true;
					break;
				case Rule::OK:
					return CORBA::string_dup(result.c_str());
					break;
				case Rule::NOTFOUND:
					throw NotFound();
			};
		};
		if(exit_now)
			break;
	};
	throw NotFound();
};
