/*
 *    FBFS Distributed Object Repository
 *    Copyright (C) 2001 Francesco V. Bassi
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _fbfs_NAMESERVER
#define _fbfs_NAMESERVER 1

#include <vector>

#include <omniORB3/CORBA.h>

#include "../idl/NameServer.hh"
#include "../utils/Prefs.h"
#include "Rule.h"

/** \file NameServer_i.h
 * Definizione della classe fbfs::NameServer_i.
 * NameServer_i  l'implementazione della interfaccia NameServer, come oggetto CORBA.
 * \ingroup NameServer
 */

namespace fbfs {
	class NameServer_i;
};

/**
 * Dato l'Obj_id di un oggetto cerca di restituire l'URI dell'ObjectManager che lo gestisce.
 * Realizza il meccanismo di gestione del sistema di nomi. Questa implementazione prevede
 * l'utilizzo di un linguaggio di definizione delle regole di ricerca e l'utilizzo eventuale
 * di altri NameServer.
 * \ingroup NameServer
 */
class fbfs::NameServer_i : public POA_fbfs::NameServer, public PortableServer::RefCountServantBase {
public:
	/**
	 * Invocato al momento della creazione dell'oggetto sigleton utilizzato dal server
	 * CORBA. Qui  inizializzato lo stato del server.
	 */
	NameServer_i(utils::Prefs &prefs, const CORBA::ORB_ptr &orb);

	/**
	 * Invocato alla chiusura del server.
	 */
	virtual ~NameServer_i() {};

	/**
	 * Restituisce l'URI del FileServer che gestisce l'oggetto che ha un determinato Obj_id.
	 * \param cert certificato di sicurezza del richiedente.
	 * \param id identificativo dell'oggetto.
	 * \return stringa C-like con l'URI del NameServer che si occupa di Obj_id.
	 * \exception NotFound non vi sono regole per risolvere il nome richiesto.
	 * \exception SecurityException il richiedente non ha i diritti per ottenere il riferimento desiderato.
	 * \exception InternalError il server non ha commesso un errore interno nel processare la richiesta.
	 */
	char * lookup(const Security &cert, const Obj_id &id)
			throw(NotFound, SecurityException, InternalError);
private:
	utils::Prefs &prefs;
	CORBA::ORB_ptr orb;
	vector<fbfs::Rule> rules;
	Security cert;
};

/** \example client.cc
 * Client per l'interrogazione di un NameServer.
 * Utilizza un file di configurazione (default "client.cfg") per mantenere
 * il valore delle variabili.
 * Ogni argomento passato a questo client viene risolto in un nome attraverso
 * l'uso di un server NameServer.
 */

/** \example resolv.cc
 * Esempio di uso del server NameServer.
 * Test con hard-coded dei valori predefiniti per le variabili.
 * Poco utile nella realt.
 */
#endif
