/*
 *    FBFS Distributed Object Repository
 *    Copyright (C) 2001 Francesco V. Bassi
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/** \file Rule.cc
 * Implementazione dei metodi della classe fbfs::Rule.
 * \ingroup NameServer
 */
   
#include "Rule.h"
#include "../utils/URI.h"
#include "../utils/NameServer.h"
#include "../idl/NameServer.hh"

#include <stdlib.h>
#include <regex.h>

using fbfs::Rule;

fbfs::Rule::Rule( int l, const string &com, const string &a) : level(l), arg(a) {
	if(com.compare("MATCH") == 0)		command = COM_MATCH;
	else if(com.compare("RESOLV") == 0)	command = COM_RESOLV;
	else if(com.compare("OK") == 0)		command = COM_OK;
	else if(com.compare("NOTFOUND") == 0) command = COM_NOTFOUND;
	else if(com.compare("FALSE") == 0)	command = COM_FALSE;
	else if(com.compare("TRUE") == 0)	command = COM_TRUE;
	else if(com.compare("LOG") == 0)	command = COM_LOG;
	else if(com.compare("RANDOM") == 0)	command = COM_RANDOM;
	else if(com.compare("EXTERN") == 0)	command = COM_EXTERN;
	else 								command = COM_UNKNOWN;
};

fbfs::Rule::evalresult fbfs::Rule::eval( const string &id, string &result, const CORBA::ORB_ptr &orb ) {
	//cout << "Eval:: command:" << command << " arg:" << arg << " level:" << level << "\n";
	switch(command) {
		case COM_FALSE:
			return FALSE;
		case COM_NOTFOUND:
			return NOTFOUND;
		case COM_OK:
			result = arg;
			return OK;
		case COM_LOG:
			cerr << "Log: [Name=" << id << "] " << arg << "\n";
			return TRUE;
		case COM_TRUE:
			return TRUE;
		case COM_RANDOM:
			if( (atof(arg.c_str()) * rand()) / RAND_MAX <= 1 )
				return TRUE;
			return FALSE;
		case COM_MATCH:
			regex_t rex;
			if(regcomp(&rex, arg.c_str(), 0)!=0) {
				cerr << "Log: [Name=" << id << "] Regular Expression error\n";
				return FALSE;
			}
			if( regexec(&rex, id.c_str(), 0, NULL,0) == 0 ) {
				regfree(&rex);
				return TRUE;
			}
			regfree(&rex);
			return FALSE;
		case COM_RESOLV:
			try {
				utils::URI uri(arg);
				CORBA::Object_var obj = utils::NameServer::get_object(orb, uri);
				NameServer_var nameserver = NameServer::_narrow(obj);
				Obj_id ident;
				ident.name = CORBA::string_dup(id.c_str());
				Security cert;
				CORBA::String_var value = nameserver->lookup(cert, ident);
				result = string(value);
				return OK;
			} catch (...) {}
			return FALSE;
		case COM_EXTERN:
			// Migliorare affinch sia possibile passare il parametro id e
			// leggere la stringa di risultato in caso di OK.
			if(!system(arg.c_str()))
				return TRUE;
			return FALSE;
		default:
			cerr << "Log: [Name=" << id << "] Unknown Command Executed\n";
			return FALSE;
	};
};		
