/*
 *    FBFS Distributed Object Repository
 *    Copyright (C) 2001 Francesco V. Bassi
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "../idl/Object.hh"

#include <string>

/** \file Object.h
 * Definizione della classe fbfs::Object
 * \ingroup Object
 */

namespace fbfs {
	class Object;
};

/**
 * Supporto per la serializzazione/deserializzazione di strutture di tipo Obj*
 * Permette la conversione in modo trasparente di strutture del tipo Obj, Obj_ts,
 * Obj_id, Obj_prop in oggetti di tipo string e viceversa.
 * 
 * Tutti i componenti del progetto devono utilizzare i metodi di questa classe
 * per la conversione.
 * \ingroup Object
 */
class fbfs::Object {
public:
	/**
	 * Serializza un struttura di tipo Obj in una string.
	 * \param obj oggetto da convertire.
	 * \result string stringa con una rappresentazione serializzata dell'oggetto obj.
	 */
	static string Obj2string(const Obj &obj);

	/**
	 * Deserializza una string in un struttura di tipo Obj.
	 * \param s stringa contenente una rappresentazione serializzata.
	 * \return oggetto corrispondente alla rappresentazione serializzata.
	 */
	static Obj *string2Obj(const string &s);

	/**
	 * Serializza un struttura di tipo Obj_prop in una string.
	 * \param obj oggetto da convertire.
	 * \result string stringa con una rappresentazione serializzata dell'oggetto obj.
	 */
	static string Obj_prop2string(const Obj_prop &obj);

	/**
	 * Deserializza una string in un struttura di tipo Obj_prop.
	 * \param s stringa contenente una rappresentazione serializzata.
	 * \return oggetto corrispondente alla rappresentazione serializzata.
	 */
	static Obj_prop string2Obj_prop(const string &s);

	/**
	 * Serializza un struttura di tipo Obj_id in una string.
	 * \param obj oggetto da convertire.
	 * \result string stringa con una rappresentazione serializzata dell'oggetto obj.
	 */
	static string Obj_id2string(const Obj_id &obj);

	/**
	 * Deserializza una string in un struttura di tipo Obj_id.
	 * \param s stringa contenente una rappresentazione serializzata.
	 * \return oggetto corrispondente alla rappresentazione serializzata.
	 */
	static Obj_id string2Obj_id(const string &s);

	/**
	 * Serializza un struttura di tipo Obj_ts in una string.
	 * \param obj oggetto da convertire.
	 * \result string stringa con una rappresentazione serializzata dell'oggetto obj.
	 */
	static string Obj_ts2string(const Obj_ts &obj);

	/**
	 * Deserializza una string in un struttura di tipo Obj_ts.
	 * \param s stringa contenente una rappresentazione serializzata.
	 * \return oggetto corrispondente alla rappresentazione serializzata.
	 */
	static Obj_ts string2Obj_ts(const string &s);

	static bool cmp_ts( const Obj_ts &uno, const Obj_ts &due);
};

/** \example objectTest1
 * Esempio d'uso delle funzionalit della classe Object.
 */

/** \example objectTest2
 * Esempio d'uso delle funzionalit della classe Object.
 */
