/*
 *    FBFS Distributed Object Repository
 *    Copyright (C) 2001 Francesco V. Bassi
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _fbfs_OBJMGR_DATA_STRUCTURES
#define _fbfs_OBJMGR_DATA_STRUCTURES

#include <list>
#include <string>
#include <map.h>

#include "../idl/Object.hh"

namespace fbfs {
	/**
	 * Informazioni relative ad una singola istanza di un oggetto. Serve per la
	 * validazione iniziale.
	 */
	struct InstanceInfo {
		string server;
		fbfs::Obj_prop prop;
		fbfs::Obj_ts ts;
	};

	/**
	 * Elenco di stringhe che rappresentano indirizzi di server. Serve sia per tenere
	 * traccia dei server attivi in un certo momento (variabile fbfs::ObjectManager_i::servers),
	 * sia per mantenere la lista di server che mantengono informazioni su un certo oggetto.
	 */
	typedef list<string> Servers;

	/**
	 * Mappa che associa al nome di un oggetto l'elenco delle InstanceInfo che lo riguardano.
	 *  il repository prima della validazione.
	 */
	typedef map<string, list<InstanceInfo>, fbfs::str_cmp> TempRepository;
	
	/**
	 * Repository nella fase successiva alla validazione: qui tutti gli Storage contengono
	 * una rappresentazione consistente del repository.
	 *
	 * E' un contenitore associativo che associa ad ogni nome di oggetto la lista dei
	 * server che lo gestiscono.
	 */
	typedef map<string, Servers, fbfs::str_cmp> Repository;
};
	
#endif
