/*
 *    FBFS Distributed Object Repository
 *    Copyright (C) 2001 Francesco V. Bassi
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _fbfs_OBJECTMANAGER
#define _fbfs_OBJECTMANAGER

#include "../idl/ObjectManager.hh"
#include "../idl/Security.hh"

#include "../utils/Prefs.h"
#include "../utils/String.h"
#include "../utils/ShareLock.h"
#include <omniORB3/CORBA.h>
#include "DataStructures.h"

/** \file ObjectManager_i.h
 * Definizione della classe fbfs::ObjectManager_i.
 * ObjectManager_i  l'implementazione della interfaccia ObjectManager, come oggetto CORBA.
 * \ingroup ObjectManager
 */

namespace fbfs {
	class ObjectManager_i;
};

/**
 * \ingroup ObjectManager
 */
class fbfs::ObjectManager_i : public POA_fbfs::ObjectManager, public PortableServer::RefCountServantBase {
public:
	/**
	 * Invocato al momento della creazione dell'oggetto sigleton utilizzato dal server
	 * CORBA. Qui  inizializzato lo stato del server.
	 */
	ObjectManager_i(const utils::Prefs &prefs, const CORBA::ORB_ptr &orb);

	/**
	 * Invocato alla chiusura del server.
	 */
	virtual ~ObjectManager_i() {};

	/**
	 */
	char * lookup(const Security &cert, const Obj_id &id)
			throw(NotFound, SecurityException, InternalError);

	/**
	 */
	void remove(const Security &cert, const Obj_id &id)
			throw(PartialExecution, NotFound, SecurityException, InternalError);

	/**
	 */
	void put(const Security &cert, const Obj_id &id, const Obj &o, const Obj_prop &p)
			throw(PartialExecution, AlreadyPresent, SecurityException, InternalError);

	/**
	 */
	void reparse(const Security &cert)
			throw(SecurityException);
private:
	void validate();
	Servers validate(const string &objname, list<InstanceInfo> &infos);
	Repository rep;
	Servers servers;
	const utils::Prefs &prefs;
	CORBA::ORB_ptr orb;
	utils::ShareLock lock;
	Security cert;
};

/** \example OMlookup.cc
 * Consulta un ObjectManager, per ricavarne l'indirizzo dello Storage contenente una
 * istanza dell'oggetto desiderato.
 */

/** \example OMput.cc
 * Introduce un oggetto nella parte di repository controllata da un determinato ObjectManager.
 */

/** \example OMremove.cc
 * Rimuove un oggetto dalla parte di repository controllata da un determinato ObjectManager.
 */

#endif
