/*
 *    FBFS Distributed Object Repository
 *    Copyright (C) 2001 Francesco V. Bassi
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/** \file objectmanager-server.cc
 * Codice di attivazione di un ObjectManager (oggetto CORBA di tipo fbfs::ObjectManager_i)
 * \ingroup ObjectManager
 */
 
#define PROJECT_NAME "fbfs"
#define MODULE_NAME "OM"

#include <iostream.h>
#include <signal.h>

#include "../utils/NameServer.h"
#include "ObjectManager_i.h"

CORBA::ORB_ptr orbptr;
const char* server_name;
const char* server_context;
const char* server_kind;

void catch_sigterm(int sig);

int main(int argc, char **argv)
{
	try {
		utils::Prefs prefs(argc, argv, "objectmanager-server.cfg");

		string context = prefs.get("context");
		server_context = context.c_str();
		string name = prefs.get("name");
		server_name = name.c_str();
		server_kind=MODULE_NAME;
		
		CORBA::ORB_var orb = CORBA::ORB_init(argc, argv, "omniORB3");

		CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
		PortableServer::POA_var poa = PortableServer::POA::_narrow(obj);

		fbfs::ObjectManager_i* object = new fbfs::ObjectManager_i(prefs, orb);

		PortableServer::ObjectId_var object_id = poa->activate_object(object);

		// Obtain a reference to the object, and register it in
		// the naming service.
		obj = object->_this();
		if( !utils::NameServer::bindObjectToName(server_context, server_name, server_kind, orb, obj) )
			return 1;

		// Now the naming service retains a reference to the object.
		// I can remove the reference obtained with object->_this();
		object->_remove_ref();

		PortableServer::POAManager_var pman = poa->the_POAManager();
		pman->activate();

		// Attiva un signal handler sul sigquit
		orbptr = orb;
		signal(SIGTERM, catch_sigterm);

		orb->run();
		orb->destroy();
	}
	catch(CORBA::SystemException&) {
		cerr << "Caught CORBA::SystemException." << endl;
	}
	catch(CORBA::Exception&) {
		cerr << "Caught CORBA::Exception." << endl;
	}
	catch(omniORB::fatalException& fe) {
		cerr << "Caught omniORB::fatalException:" << endl;
		cerr << "  file: " << fe.file() << endl;
		cerr << "  line: " << fe.line() << endl;
		cerr << "  mesg: " << fe.errmsg() << endl;
	}
	catch(...) {
		cerr << "Caught unknown exception." << endl;
	}

	return 0;
}

void catch_sigterm( int sig ) {
	utils::NameServer::unbindObject(server_context, server_name, server_kind, orbptr);
	orbptr->shutdown(true);	
}
