/*
 *    FBFS Distributed Object Repository
 *    Copyright (C) 2001 Francesco V. Bassi
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/** \file ObjectManager_i.cc
 * Implementazione dei metodi della classe fbfs::ObjectManager_i
 * \ingroup ObjectManager
 */
 
#include "ObjectManager_i.h"
#include "../utils/URI.h"
#include "../utils/NameServer.h"
#include "../utils/TimeStamp.h"
#include "../idl/Storage.hh"

using namespace fbfs;
using namespace utils;

fbfs::ObjectManager_i::ObjectManager_i(const utils::Prefs &p, const CORBA::ORB_ptr &o)
		: prefs(p), orb(o) {
};

char *fbfs::ObjectManager_i::lookup(const Security &cert, const Obj_id &id)
		throw(NotFound, SecurityException, InternalError) {
	return strdup("fbfs://Storage@contesto/nome");
};

void fbfs::ObjectManager_i::remove(const Security &cert, const Obj_id &id)
		throw(NotFound, SecurityException, InternalError) {

	utils::URI uri("fbfs://Storage@contesto/nome");
	CORBA::Object_var obj = utils::NameServer::get_object(orb, uri);
	fbfs::Storage_var server = fbfs::Storage::_narrow(obj);
	if(CORBA::is_nil(server)) {
		cerr << "is_nil(server) == TRUE.\n";
		exit(1);
	};
	
	server->remove(cert, id);
};

void fbfs::ObjectManager_i::put(const Security &cert, const Obj_id &id, const Obj &o, const Obj_prop &p)
		throw(AlreadyPresent, SecurityException, InternalError) {

	utils::URI uri("fbfs://Storage@contesto/nome");
	CORBA::Object_var obj = utils::NameServer::get_object(orb, uri);
	fbfs::Storage_var server = fbfs::Storage::_narrow(obj);
	if(CORBA::is_nil(server)) {
		cerr << "is_nil(server) == TRUE.\n";
		exit(1);
	};
	
	Obj_ts ts;
	ts.time = utils::TimeStamp::current();
	
	server->put(cert, o, id, p, ts);
};

void fbfs::ObjectManager::reparse(const Security &cert) throw(SecurityException) {
};
