/*
 *    FBFS Distributed Object Repository
 *    Copyright (C) 2001 Francesco V. Bassi
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "File.h"
#include "File.h"

#include <strstream>

#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

/** \file File.cc
 * Implementazione dei metodi della classe utils::File.
 * \ingroup Utils
 */

void utils::File::write(const string &s, const string &name, const string &dir, bool always=false)
				throw(MEMError, IOError, AlreadyPresent) {
    string fullname = name;
	if(dir.length() != 0)
        fullname = dir + "/" + fullname;
	
	int fd;
	if(always)
    	fd = open(fullname.c_str(), O_CREAT | O_TRUNC | O_WRONLY, 0664);
	else
    	fd = open(fullname.c_str(), O_CREAT | O_EXCL | O_WRONLY, 0664);

    if(fd < 0)
        switch(errno) {
			case EEXIST:
                throw AlreadyPresent();
            default:
                throw IOError();
        };
    int nbytes;

	const char * cs = s.c_str();
	size_t len = s.length();
	
    do {
        nbytes = ::write(fd, cs, len);
    } while (nbytes == -1 && errno == EINTR);

    close(fd);
    if(nbytes < 0) {
        unlink(name.c_str());
        throw IOError();
	}
};

string utils::File::read(const string &name, const string &dir) throw(MEMError, IOError, NotFound) {
    string fullname = name;
	if(dir.length() != 0)
        fullname = dir + "/" + fullname;

    int fd = open(fullname.c_str(), 0);
    if(fd < 0)
        switch(errno) {
			case ENOENT:
                throw NotFound();
            default:
                throw IOError();
        };
	
	string s;
	char c;
	int k=0;
	while(-1) {
		int i = ::read(fd, &c, 1);
		if(i==0)
			break;
		if(i == -1 && errno == EINTR)
			continue;
		if(i<0) {
			close(fd);
			throw IOError();
		};
		s += c;
	};
	close(fd);
	return s;
};

void utils::File::remove(const string &name, const string &dir) throw(IOError, NotFound) {
    string fullname = name;
	if(dir.length() != 0)
        fullname = dir + "/" + fullname;

	int res = unlink(fullname.c_str());
	if(res == 0)
		return;
	switch(errno) {
		case ENOENT:
			throw NotFound();
		default:
			throw IOError();
	};
};	  
