/*
 *    FBFS Distributed Object Repository
 *    Copyright (C) 2001 Francesco V. Bassi
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _utils_FILE
#define _utils_FILE 1

#include <string>

/** \file File.h
 * Definizione della classe utils::File.
 * \ingroup Utils
 */

namespace utils {
	class File;
};

/**
 * Fornisce un approccio conveniente per la gestione dei file.
 * I metodi forniti da questa classe forniscono un livello di astrazione all'accesso
 * ai file, rendendo platform-independent i server che utilizzano esclusivamente
 * questa classe per l'accesso al filesystem.
 * \ingroup Utils
 */
class utils::File {
	public:
			class Exception {};
			class IOError : Exception {};
			class MEMError : Exception {};
			class AlreadyPresent : Exception {};
			class NotFound : IOError {};
			
			static void write(const string &s, const string &name, const string &dir, bool always=false)
					throw(MEMError, IOError, AlreadyPresent);
			static string read(const string &name, const string &dir)
					throw(MEMError, IOError, NotFound);
			static void remove(const string &name, const string &dir)
					throw(IOError, NotFound);
};

#endif
