/*
 *    FBFS Distributed Object Repository
 *    Copyright (C) 2001 Francesco V. Bassi
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/** \file Lock.cc
 * Implementazione dei metodi della classe utils::Lock.
 * \ingroup Utils
 */

#include "Lock.h"

utils::Lock::Lock() {
	pthread_mutex_init(&mut, NULL);
	pthread_cond_init(&cond, NULL);
};

utils::Lock::~Lock() {
	pthread_mutex_destroy(&mut);
	pthread_cond_destroy(&cond);
};

inline void utils::Lock::alloc(const string &res) { resources.insert(res); };
inline void utils::Lock::free(const string &res) { resources.erase(res); };
inline bool utils::Lock::isfree(const string &res) { return resources.count(res)==0; };

void utils::Lock::lock(const string &resource) {
	pthread_mutex_lock(&mut);
	while(!isfree(resource))
		pthread_cond_wait(&cond, &mut);
	alloc(resource);
	pthread_mutex_unlock(&mut);
};

void utils::Lock::unlock(const string &resource) {
	pthread_mutex_lock(&mut);
	free(resource);
	pthread_cond_broadcast(&cond);
	pthread_mutex_unlock(&mut);
};

