/*
 *    FBFS Distributed Object Repository
 *    Copyright (C) 2001 Francesco V. Bassi
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "Prefs.h"

#include <string.h>

/** \file Prefs.cc
 * Implementazione dei metodi della classe utils::Prefs.
 * \ingroup Utils
 */

utils::Prefs::Prefs(int &argc, char **&argv, string filename = "") {
	char **nargv = (char **) calloc(argc+2, sizeof(char*));
	nargv[0]=argv[0];
	int i=1,j=1;

	while(i<argc) {
		if(strcmp(argv[i],"-config")==0 && i<argc-1) {
			filename = argv[i+1];
			i += 2;
		} else {
			nargv[j++] = argv[i++];
		};
	};
	nargv[j+1] = NULL;
	argc = j;
	argv = nargv;
	
	accessible = Prefs::access(filename);
};

utils::Prefs::Prefs(string filename) {
	accessible = Prefs::access(filename);
};

bool utils::Prefs::access(string filename) {
	file.open(filename.c_str());
	return file.is_open();
};
	
string utils::Prefs::get(const string &tag, const string defvalue = "") const {
	if(!accessible)
		return defvalue;
	char buf[80];
	string key;
	string value;
	file.seekg(0, ios::beg);
	do {
		file.getline(buf,80);
		string s(buf);
		int i = s.find(" ");
		key = s.substr(0,i);
		value = s.substr(i+1,s.length()-i-1);
		if(key.compare(tag)==0)
			return value;
	} while (!file.eof());
	return defvalue;
};
