/*
 *    FBFS Distributed Object Repository
 *    Copyright (C) 2001 Francesco V. Bassi
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _utils_PREFS
#define _utils_PREFS 1

#include <string>
#include <fstream.h>

/** \file Prefs.h
 * Definizione della classe utils::Prefs.
 * \ingroup Utils
 */

namespace utils {
	class Prefs;
};

/**
 * Permette di accedere in modo conveniente a dei file di configurazione appositamente
 * realizzati.
 * \ingroup Utils
 */
class utils::Prefs {
	public:
			/**
			 * Crea un oggetto di tipo Prefs effettuando il parsing degli argomenti
			 * passati come parametri di invocazione standard C.
			 * Verifica tra i parametri la presenza di: "-config <filename>" e qualora
			 * essa sia presente, la rimuove dalla lista dei parametri e utilizza
			 * <filename> come file di configurazione.
			 * \param argc numero argomenti
			 * \param argv puntatore ad un array di argomenti
			 * \param defaultfile nome del file di configurazione qualora tra gli argomenti
			 * non sia passato.
			 *
			 * Qualora il file di configurazione non sia presente, l'oggetto viene creato
			 * comunque.
			 */
			Prefs(int &argc, char **&argv, string defaultfile = "");

			/**
			 * Crea un oggetto di tipo Prefs specificando esplicitamente il nome del file
			 * da utilizzare.
			 * \param filename nome del file di configurazione
			 */
			Prefs(string filename);

			/**
			 * Verifica se l'oggetto si riferisce ad un file di configurazione accessibile.
			 * \return true se il file di configurazione esiste ed  accessibile.
			 */
			bool is_accessible() const {return accessible;};

			/**
			 * Restituisce il valore associato alla chiave desiderata.
			 * \param tag chiave da ricercare nel file di configurazione.
			 * \return valore associato alla chiave tag.
			 */
			string get(const string &tag, const string defvalue = "") const;
	private:
			mutable ifstream::ifstream file;
			bool accessible;
			bool access(string file);
};

#endif
