/*
 *    FBFS Distributed Object Repository
 *    Copyright (C) 2001 Francesco V. Bassi
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/** \file Lock.cc
 * Implementazione dei metodi della classe utils::Lock.
 * \ingroup Utils
 */

#include "ShareLock.h"

utils::ShareLock::ShareLock() {
	pthread_mutex_init(&mut, NULL);
	pthread_cond_init(&cond, NULL);
};

utils::ShareLock::~ShareLock() {
	pthread_mutex_destroy(&mut);
	pthread_cond_destroy(&cond);
};

inline void utils::ShareLock::alloc(const string &res, bool ex) {
	if(ex)
		excl.insert(res);
	else
		shared.insert(res);
};

inline void utils::ShareLock::free(const string &res) {
	if(shared.count(res)==0)
		excl.erase(res);
	else
		shared.erase(res);
};

inline bool utils::ShareLock::isbusy(const string &res, bool ex) {
	return excl.count(res)!=0 || (ex && (shared.count(res)!=0) );
};

void utils::ShareLock::lock(const string &resource, bool ex=false) {
	pthread_mutex_lock(&mut);
	while(isbusy(resource, ex))
		pthread_cond_wait(&cond, &mut);
	alloc(resource, ex);
	pthread_mutex_unlock(&mut);
};

void utils::ShareLock::unlock(const string &resource) {
	pthread_mutex_lock(&mut);
	free(resource);
	pthread_cond_broadcast(&cond);
	pthread_mutex_unlock(&mut);
};

