/*
 *    FBFS Distributed Object Repository
 *    Copyright (C) 2001 Francesco V. Bassi
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "URI.h"

#define DEFAULT_PORT "2809"
#define DEFAULT_HOST "localhost"

/** \file URI.cc
 * Implementazione dei metodi della classe utils::URI
 * \ingroup Utils
 */

utils::URI::URI( const string &s ) {
	_isValid = false;

	string t;
	string::size_type l;
	
	if( s.find("fbfs:/") != 0 )
		return;
	t = s.substr(6, s.length()-6);
	l = t.find("/");
	if( l == string::npos )
		return;
	string address = t.substr(0,l);
	string service = t.substr(l+1,t.length()-l-1);
	l = address.find(":");
	if( l == string::npos ) {
		_host = address;
		_port = DEFAULT_PORT;
	} else {
		_host = address.substr(0,l);
		_port = address.substr(l+1,address.length()-l-1);
	};
	if( _host.length() == 0 )
		_host = DEFAULT_HOST;
	l = service.find("@");
	if( l == string::npos )
		return;
	_service = service.substr(0,l);
	t = service.substr(l+1,service.length()-l-1);
	l = t.find("/");
	if( l == string::npos )
		return;
	_context = t.substr(0,l);
	_name = t.substr(l+1,t.length()-l-1);
	_isValid = true;
};

string utils::URI::nameServerURL() const {
	return "corbaloc::" + _host + ":" + _port + "/NameService";
};

string utils::URI::serviceName() const {
	return _context + ".fbfs/" + _name + "." + _service;
};

string utils::URI::toString() const {
	if(_isValid)
		return "fbfs://" + _host + ":" + _port + "/" + _service + "@" + _context + "/" + _name;
	else
		return "";
};
